@@ -1,14 +1,21 @@
+# Copyright 2002 Ben Escoto
+#
+# This file is part of rdiff-backup.
+#
+# rdiff-backup is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge MA
+# 02139, USA; either version 2 of the License, or (at your option) any
+# later version; incorporated herein by reference.
+
+"""Support code for remote execution and data transfer"""
+
 from __future__ import generators
 import types, os, tempfile, cPickle, shutil, traceback, pickle
 
-#######################################################################
-#
-# connection - Code that deals with remote execution
-#
 
 class ConnectionError(Exception): pass
 class ConnectionReadError(ConnectionError): pass
-
 class ConnectionQuit(Exception): pass
 
 
