@@ -37,7 +37,7 @@ able to narrow down the possibilities.
 
 """
 
-import tempfile, errno, signal, cPickle, C
+import tempfile, errno, signal, cPickle, C, pickle
 from static import *
 
 
@@ -254,7 +254,7 @@ class Robust:
 		try: return function(*args)
  		except (EnvironmentError, SkipFileException, DSRPPermError,
 				RPathException, Rdiff.RdiffException,
-				C.UnknownFileTypeError), exc:
+				librsync.librsyncError, C.UnknownFileTypeError), exc:
 			TracebackArchive.add()
 			if (not isinstance(exc, EnvironmentError) or
 				(errno.errorcode[exc[0]] in
@@ -264,8 +264,8 @@ class Robust:
 				Log.exception()
 				conn = Globals.backup_writer
 				if conn is not None: # increment error count
-					ITR_exists = conn.Globals.is_not_None('ITR')
-					if ITR_exists: conn.Globals.ITR.increment_stat('Errors')
+					ITRB_exists = conn.Globals.is_not_None('ITRB')
+					if ITRB_exists: conn.Globals.ITRB.increment_stat('Errors')
 				if error_handler: return error_handler(exc, *args)
 			else:
 				Log.exception(1, 2)
@@ -598,7 +598,7 @@ class Resume:
 		fp = checkpoint_rp.open("rb")
 		data = fp.read()
 		fp.close()
-		return cPickle.loads(data)
+		return pickle.loads(data)
 
 	def ResumeCheck(cls):
 		"""Return relevant ResumeSessionInfo if there's one we should resume
@@ -659,5 +659,5 @@ class ResumeSessionInfo:
 
 from log import *
 from destructive_stepping import *
-import Time, Rdiff
+import Time, Rdiff, librsync
 from highlevel import *
